﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Uno.Extensions;
using Uno.Logging;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Media;
using Uno.Disposables;
using Windows.UI.Core;

#if __IOS__
using UIKit;
using _Image = UIKit.UIImage;
#elif __MACOS__
using AppKit;
using _Image = AppKit.NSImage;
#endif

namespace Windows.UI.Xaml.Media
{
	public partial class ImageBrush
	{
		private readonly SerialDisposable _imageScheduler = new SerialDisposable();

		internal event Action<_Image> ImageChanged;

		partial void OnSourceChangedPartial(ImageSource newValue, ImageSource oldValue)
		{
			if (newValue == null || !newValue.HasSource())
			{
				SetImage(null, failIfNull: false);
				oldValue?.Dispose();
				_imageScheduler.Disposable = null;
			}
			else if (newValue.TryOpenSync(out var image))
			{
				SetImage(image);
				oldValue?.Dispose();
				_imageScheduler.Disposable = null;
			}
			else
			{
				SetImage(null, failIfNull: false);
				oldValue?.Dispose();
				OpenImageAsync(newValue);
			}
		}

		private async void OpenImageAsync(ImageSource newValue)
		{
			CoreDispatcher.CheckThreadAccess();

			using var cd = new CancellationDisposable();
			_imageScheduler.Disposable = cd;

			var image = await Task.Run(() => newValue.Open(cd.Token), cd.Token);

			if (cd.Token.IsCancellationRequested)
			{
				return;
			}

			SetImage(image);
		}

		private void SetImage(_Image image,  bool failIfNull = true)
		{
			ImageChanged?.Invoke(image);

			if (image != null)
			{
				OnImageOpened();
			}
			else if (failIfNull)
			{
				OnImageFailed();
			}
		}
	}
}
